@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2021 Mike Fährmann
+# Copyright 2016-2025 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -15,7 +15,7 @@ from .. import text, exception
 class LusciousExtractor(Extractor):
     """Base class for luscious extractors"""
     category = "luscious"
-    cookiedomain = ".luscious.net"
+    cookies_domain = ".luscious.net"
     root = "https://members.luscious.net"
 
     def _graphql(self, op, variables, query):
@@ -26,15 +26,15 @@ class LusciousExtractor(Extractor):
             "variables"    : variables,
         }
         response = self.request(
-            "{}/graphql/nobatch/?operationName={}".format(self.root, op),
+            f"{self.root}/graphql/nobatch/?operationName={op}",
             method="POST", json=data, fatal=False,
         )
 
         if response.status_code >= 400:
             self.log.debug("Server response: %s", response.text)
-            raise exception.StopExtraction(
-                "GraphQL query failed ('%s %s')",
-                response.status_code, response.reason)
+            raise exception.AbortExtraction(
+                f"GraphQL query failed "
+                f"('{response.status_code} {response.reason}')")
 
         return response.json()["data"]
 
@@ -47,87 +47,29 @@ class LusciousAlbumExtractor(LusciousExtractor):
     archive_fmt = "{album[id]}_{id}"
     pattern = (r"(?:https?://)?(?:www\.|members\.)?luscious\.net"
                r"/(?:albums|pictures/c/[^/?#]+/album)/[^/?#]+_(\d+)")
-    test = (
-        ("https://luscious.net/albums/okinami-no-koigokoro_277031/", {
-            "url": "7e4984a271a1072ac6483e4228a045895aff86f3",
-            #  "content": "b3a747a6464509440bd0ff6d1267e6959f8d6ff3",
-            "keyword": {
-                "album": {
-                    "__typename"  : "Album",
-                    "audiences"   : list,
-                    "content"     : "Hentai",
-                    "cover"       : "re:https://\\w+.luscious.net/.+/277031/",
-                    "created"     : 1479625853,
-                    "created_by"  : "NTRshouldbeillegal",
-                    "date"        : "dt:2016-11-20 07:10:53",
-                    "description" : "Enjoy.",
-                    "download_url": "re:/download/(r/)?824778/277031/",
-                    "genres"      : list,
-                    "id"          : 277031,
-                    "is_manga"    : True,
-                    "labels"      : list,
-                    "language"    : "English",
-                    "like_status" : "none",
-                    "modified"    : int,
-                    "permissions" : list,
-                    "rating"      : float,
-                    "slug"        : "okinami-no-koigokoro",
-                    "status"      : str,
-                    "tags"        : list,
-                    "title"       : "Okinami no Koigokoro",
-                    "url"         : "/albums/okinami-no-koigokoro_277031/",
-                    "marked_for_deletion": False,
-                    "marked_for_processing": False,
-                    "number_of_animated_pictures": 0,
-                    "number_of_favorites": int,
-                    "number_of_pictures": 18,
-                },
-                "aspect_ratio": r"re:\d+:\d+",
-                "category"    : "luscious",
-                "created"     : int,
-                "date"        : "type:datetime",
-                "height"      : int,
-                "id"          : int,
-                "is_animated" : False,
-                "like_status" : "none",
-                "position"    : int,
-                "resolution"  : r"re:\d+x\d+",
-                "status"      : str,
-                "tags"        : list,
-                "thumbnail"   : str,
-                "title"       : str,
-                "width"       : int,
-                "number_of_comments": int,
-                "number_of_favorites": int,
-            },
-        }),
-        ("https://luscious.net/albums/not-found_277035/", {
-            "exception": exception.NotFoundError,
-        }),
-        ("https://members.luscious.net/albums/login-required_323871/", {
-            "count": 64,
-        }),
-        ("https://www.luscious.net/albums/okinami_277031/"),
-        ("https://members.luscious.net/albums/okinami_277031/"),
-        ("https://luscious.net/pictures/c/video_game_manga/album"
-         "/okinami-no-koigokoro_277031/sorted/position/id/16528978/@_1"),
-    )
+    example = "https://luscious.net/albums/TITLE_12345/"
 
     def __init__(self, match):
         LusciousExtractor.__init__(self, match)
-        self.album_id = match.group(1)
+        self.album_id = match[1]
+
+    def _init(self):
         self.gif = self.config("gif", False)
 
     def items(self):
         album = self.metadata()
-        yield Message.Directory, {"album": album}
+        yield Message.Directory, "", {"album": album}
         for num, image in enumerate(self.images(), 1):
             image["num"] = num
             image["album"] = album
 
-            image["thumbnail"] = image.pop("thumbnails")[0]["url"]
+            try:
+                image["thumbnail"] = image.pop("thumbnails")[0]["url"]
+            except LookupError:
+                image["thumbnail"] = ""
+
             image["tags"] = [item["text"] for item in image["tags"]]
-            image["date"] = text.parse_timestamp(image["created"])
+            image["date"] = self.parse_timestamp(image["created"])
             image["id"] = text.parse_int(image["id"])
 
             url = (image["url_to_original"] or image["url_to_video"]
@@ -246,7 +188,7 @@ fragment AlbumStandard on Album {
         album["created_by"] = album["created_by"]["display_name"]
 
         album["id"] = text.parse_int(album["id"])
-        album["date"] = text.parse_timestamp(album["created"])
+        album["date"] = self.parse_timestamp(album["created"])
 
         return album
 
@@ -334,19 +276,11 @@ class LusciousSearchExtractor(LusciousExtractor):
     subcategory = "search"
     pattern = (r"(?:https?://)?(?:www\.|members\.)?luscious\.net"
                r"/albums/list/?(?:\?([^#]+))?")
-    test = (
-        ("https://members.luscious.net/albums/list/"),
-        ("https://members.luscious.net/albums/list/"
-         "?display=date_newest&language_ids=%2B1&tagged=+full_color&page=1", {
-             "pattern": LusciousAlbumExtractor.pattern,
-             "range": "41-60",
-             "count": 20,
-         }),
-    )
+    example = "https://luscious.net/albums/list/?tagged=TAG"
 
     def __init__(self, match):
         LusciousExtractor.__init__(self, match)
-        self.query = match.group(1)
+        self.query = match[1]
 
     def items(self):
         query = text.parse_query(self.query)
